@@ -353,7 +353,15 @@ sub compile_create {
 
     my $meta_instance = $meta->get_meta_instance;
 
-    return sub { $meta_instance->create_instance() };
+    my $cache = does_role($meta, "KiokuDB::Role::Cacheable");
+
+    my @register_args = (
+        ( $cache ? ( cache => 1 ) : () ),
+    );
+
+    return sub {
+        return ( $meta_instance->create_instance(), @register_args );
+    };
 }
 
 sub compile_clear {
